@@ -19,7 +19,7 @@
  */
 
 #include "dialog_connection.h"
-#include "box_db.h" //For Box_DB::connect_to_server().
+#include "box_withbuttons.h" //For Box_WithButtons::connect_to_server().
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -37,13 +37,36 @@ Dialog_Connection::Dialog_Connection(BaseObjectType* cobject, const Glib::RefPtr
   refGlade->get_widget("entry_user", m_entry_user);
   refGlade->get_widget("entry_password", m_entry_password);
   refGlade->get_widget("label_database", m_label_database);
+
+#ifdef GLOM_ENABLE_MAEMO
+  // Make the bold title the window title (which cannot be empty in maemo
+  // because it displays <Untitled window> instead). This also helps to
+  // make the dialog smaller in height, so we save a bit screen space required
+  // by the onscreen keyboard.
+  Gtk::Label* title;
+  Gtk::Label* note;
+
+  refGlade->get_widget("connection_title", title);
+  refGlade->get_widget("connection_note", note);
+
+  set_title(title->get_text());
+  title->hide();
+
+  // Without size request, this label enlarges the dialog significantly,
+  // and the text is still truncated.
+  note->set_size_request(400, -1);
+#endif
 }
 
 Dialog_Connection::~Dialog_Connection()
 {
 }
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
 sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection_settings() const
+#else
+sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection_settings(std::auto_ptr<ExceptionConnection>& error) const
+#endif
 {
   //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings()" << std::endl;
 
@@ -57,24 +80,50 @@ sharedptr<SharedConnection> Dialog_Connection::connect_to_server_with_connection
     //Set the connection details in the ConnectionPool singleton.
     //The ConnectionPool will now use these every time it tries to connect.
 
-    //const Document_Glom* document = get_document();
-    //if(document)
-    //{
+    const Document_Glom* document = get_document();
+    if(document)
+    {
       //std::cout << "debug: Dialog_Connection::connect_to_server_with_connection_settings(): m_database_name=" << m_database_name << std::endl;
       connection_pool->set_database(m_database_name);
 
-      connection_pool->set_host(m_entry_host->get_text());
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+      if(document->get_connection_is_self_hosted())
+        connection_pool->set_host("localhost");
+      else
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+        connection_pool->set_host(m_entry_host->get_text());
+ 
       connection_pool->set_user(m_entry_user->get_text());
       connection_pool->set_password(m_entry_password->get_text());
+
+      //Start with the same port as last time the document was used:
+      //and avoid trying other ports if this is a self-hosted or network-browsed database.
+      connection_pool->set_port(document->get_connection_port());
+      connection_pool->set_try_other_ports(document->get_connection_try_other_ports());
+
+      
       //if(document)
       //{
       //  connection_pool->set_database(document->get_connection_database());
       //}
-    //}
+    }
 
-    connection_pool->set_ready_to_connect(); //Box_DB::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
+    connection_pool->set_ready_to_connect(); //Box_WithButtons::connect_to_server() will now attempt the connection-> Shared instances of m_Connection will also be usable.
 
-    result = Box_DB::connect_to_server(const_cast<Dialog_Connection*>(this));
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this));
+#else
+    result = Base_DB::connect_to_server(const_cast<Dialog_Connection*>(this), error);
+#endif
+
+    if(document)
+    {
+      //Remember the port, 
+      //to make opening faster next time,
+      //and so we can tell connecting clients (using browse network) what port to use:
+      Document_Glom* unconst = const_cast<Document_Glom*>(document);
+      unconst->set_connection_port( connection_pool->get_port() );
+    }
 
     /*
     if(document)
@@ -95,11 +144,23 @@ void Dialog_Connection::load_from_document()
   Document_Glom* document = get_document();
   if(document)
   {
+#ifndef GLOM_ENABLE_CLIENT_ONLY
     //Load server and user:
-    Glib::ustring host = document->get_connection_server();
-    if(host.empty())
-      host = "localhost";
-    m_entry_host->set_text(host);
+    if(document->get_connection_is_self_hosted())
+    {
+       m_entry_host->set_text("(self hosted)");
+       m_entry_host->set_sensitive(false);
+    }
+    else
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+    {
+      Glib::ustring host = document->get_connection_server();
+      if(host.empty())
+        host = "localhost";
+     
+      m_entry_host->set_text(host);
+      m_entry_host->set_sensitive(true);
+    }
 
     Glib::ustring user = document->get_connection_user(); //TODO: Offer a drop-down list of users.
 
@@ -122,6 +183,28 @@ void Dialog_Connection::load_from_document()
 
 }
 
+///Disable irrelevant fields:
+void Dialog_Connection::set_connect_to_browsed()
+{
+  m_entry_host->set_sensitive(false);
+}
+
+void Dialog_Connection::set_self_hosted_user_and_password(const Glib::ustring& user, const Glib::ustring& password)
+{
+  set_username(user);
+  set_password(password);
+}
+
+void Dialog_Connection::set_username(const Glib::ustring& user)
+{
+  m_entry_user->set_text(user);
+}
+
+void Dialog_Connection::set_password(const Glib::ustring& password)
+{
+  m_entry_password->set_text(password);
+}
+
 void Dialog_Connection::set_database_name(const Glib::ustring& name)
 {
   m_database_name = name;
@@ -131,4 +214,10 @@ void Dialog_Connection::set_database_name(const Glib::ustring& name)
     m_label_database->set_text(name);
 }
 
+void Dialog_Connection::get_username_and_password(Glib::ustring& username, Glib::ustring& password) const
+{
+  username = m_entry_user->get_text();
+  password = m_entry_password->get_text();
+}
+
 } //namespace Glom
