@@ -19,7 +19,7 @@
  */
  
 #include "dialog_database_preferences.h"
-#include "box_db.h" //For Box_DB::connect_to_server().
+#include "box_withbuttons.h" //For Box_WithButtons::connect_to_server().
 #include <glom/libglom/standard_table_prefs_fields.h>
 #include <glom/libglom/data_structure/glomconversions.h>
 #include <bakery/Utilities/BusyCursor.h>
@@ -145,6 +145,12 @@ void Dialog_Database_Preferences::load_from_document()
   NumericFormat numeric_format; //ignored
 
   Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, this);
+  if(!datamodel)
+  {
+    std::cerr << "Dialog_Database_Preferences::load_from_document(): Gda::DataModel is NULL." << std::endl;
+    return;
+  }
+
   guint count = datamodel->get_n_rows();
   for(guint i = 0; i < count; ++i)
   {
