@@ -18,6 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
+#include "config.h" // For HAVE_STRPTIME
+
 #include "glomconversions.h"
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/utils.h>
@@ -236,20 +239,15 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 
     return format_time(the_c_time, locale, iso_format);
   }
-  else if( (glom_type == Field::TYPE_NUMERIC) && (value.get_value_type() == GDA_TYPE_NUMERIC))
+  else if(glom_type == Field::TYPE_NUMERIC)
   {
-    const GdaNumeric* gda_numeric = value.get_numeric();
-    std::string text_in_c_locale;
-    if(gda_numeric && gda_numeric->number) //A char* - I assume that it formatted as per the C locale. murrayc. TODO: Do we need to look at the other fields?
-      text_in_c_locale = gda_numeric->number; //What formatting does this use?
+    if(value.get_value_type() != GDA_TYPE_NUMERIC)
+    {
+      std::cerr << "Conversions::get_text_for_gda_value(): glom field type is NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
+      return value.to_string();
+    }
 
-    //Get an actual numeric value, so we can get a locale-specific text representation:
-    std::stringstream the_stream;
-    the_stream.imbue( std::locale::classic() ); //The C locale.
-    the_stream.str(text_in_c_locale); //Avoid using << because Glib::ustinrg does implicit character conversion with that.
-
-    double number = 0;
-    the_stream >> number;
+    const double number = get_double_for_gda_value_numeric(value);
 
     //Get the locale-specific text representation, in the required format:
     std::stringstream another_stream;
@@ -311,7 +309,7 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
   }
   else
   {
-    std::cerr << "Conversions::get_text_for_gda_value(): Unexpected glom field type: " << glom_type << ", gdatype=" << g_type_name(value.get_value_type()) << std::endl;
+    std::cerr << "Conversions::get_text_for_gda_value(): Unexpected glom field type: " << glom_type << std::endl;
     return value.to_string();
   }
 }
@@ -440,12 +438,12 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
   }
   else if(glom_type == Field::TYPE_IMAGE)
   {
-    //We assume that the text is the same (escaped text) format that we use in the document when saving examples.
+    //We assume that the text is the same (escaped text) format that we use in the document when saving images:
     //(The SQL format).
     Gnome::Gda::Value result;
 
     size_t buffer_binary_length = 0;
-    guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
+    guchar* buffer_binary = Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
     if(buffer_binary)
     {
       result.set(buffer_binary, buffer_binary_length);
@@ -642,6 +640,7 @@ tm Conversions::parse_time(const Glib::ustring& text, const std::locale& locale,
     return the_c_time;
   }
   
+#ifdef HAVE_STRPTIME
   //Fall back to strptime():
   //It seems to be well known that time_get<> can parse much less than what time_put<> can generate:
   // http://www.open-std.org/jtc1/sc22/wg21/docs/papers/2006/n2070.html
@@ -688,6 +687,7 @@ tm Conversions::parse_time(const Glib::ustring& text, const std::locale& locale,
  
   //std::cout << "  DEBUG: strptime(%c) failed on text=" << text << std::endl;
 
+#endif // HAVE_STRPTIME
   //Nothing worked:
   //tm blank_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
   tm blank_time ;
@@ -1079,7 +1079,7 @@ Glib::RefPtr<Gdk::Pixbuf> Conversions::get_pixbuf_for_gda_value(const Gnome::Gda
         {
           const Glib::Exception& ex = *error.get();        
 #endif
-          g_warning("ImageGlom::set_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
+          g_warning("Conversions::get_pixbuf_for_gda_value(): PixbufLoader::write() failed: %s", ex.what().c_str());
         }
       }
 
