@@ -36,14 +36,17 @@
 #include <glom/libglom/data_structure/tableinfo.h>
 #include <glom/libglom/data_structure/groupinfo.h>
 #include <glom/libglom/data_structure/report.h>
+#include <glom/libglom/data_structure/print_layout.h>
 #include "../appstate.h"
 #include <gtkmm/window.h>
 #include <vector>
 #include <map>
+#include <limits> // for numeric_limits
 
 namespace Glom
 {
 
+/// Can be used with std::find_if() to find a layout with the same parent_table and layout_name.
 template<class T_Element>
 class predicate_Layout
 {
@@ -80,6 +83,12 @@ public:
 
   virtual void set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension = false);
 
+  /** Whether the document was opened from another networked glom instance, 
+   * instead of via a URI.
+   */
+  void set_opened_from_browse(bool val = true);
+  bool get_opened_from_browse() const;
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   /** The document usually saves itself when you call set_modified().
    * Pass false to this function to prevent that temporarily.
@@ -112,6 +121,8 @@ public:
   void set_connection_server(const Glib::ustring& strVal);
   void set_connection_user(const Glib::ustring& strVal);
   void set_connection_database(const Glib::ustring& strVal);
+  void set_connection_port(int port_number);
+  void set_connection_try_other_ports(bool val);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   /** When this returns true, the postgres database should be hosted by the local client,
@@ -129,6 +140,8 @@ public:
   Glib::ustring get_connection_server() const;
   Glib::ustring get_connection_user() const;
   Glib::ustring get_connection_database() const;
+  int get_connection_port() const;
+  bool get_connection_try_other_ports() const;
 
   /** Set the language/locale used by original titles.
    * Title translations are translations of the text in this language.
@@ -173,25 +186,43 @@ public:
   void remove_field(const Glib::ustring& table_name, const Glib::ustring& field_name);
 
 
-  typedef std::map<guint, sharedptr<LayoutGroup> > type_mapLayoutGroupSequence;
-  type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
+  typedef std::vector< sharedptr<LayoutGroup> > type_list_layout_groups;
 
-  void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups);
+  /** Get the layout groups for a layout.
+   * @param layout_name The name of the layout, such as list or details.
+   * @param parent_table_name The name of the table for which this layout should appear.
+   * @result A list of layout groups at the top-level of the requested layout.
+   */
+  type_list_layout_groups get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
+
+  /** Discover whether there are any fields in the layout.
+   * @param layout_name The name of the layout, such as list or details.
+   * @param parent_table_name The name of the table for which this layout should appear.
+   * @result true if there is at least one field in the layout group or its sub groups.
+   */
+  bool get_data_layout_groups_have_any_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
+
+  /** Set the layout groups for a layout.
+   * @param layout_name The name of the layout, such as list or details.
+   * @param parent_table_name The name of the table for which this layout should appear.
+   * @param groups A list of layout groups at the top-level of the requested layout.
+   */
+  void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_list_layout_groups& groups);
 
   /**
    * @para The layout_name, such as "details", "list".
    * @para parent_table_name The name of the table on whose layout the layout appears.
    */
-  type_mapLayoutGroupSequence get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
+  type_list_layout_groups get_data_layout_groups_plus_new_fields(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
 
-  type_mapLayoutGroupSequence get_data_layout_groups_default(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
+  type_list_layout_groups get_data_layout_groups_default(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
 
   typedef std::list< sharedptr<TranslatableItem> > type_list_translatables;
   type_list_translatables get_translatable_layout_items(const Glib::ustring& table_name);
   type_list_translatables get_translatable_report_items(const Glib::ustring& table_name, const Glib::ustring& report_title);
 
   void fill_layout_field_details(const Glib::ustring& parent_table_name, const sharedptr<LayoutGroup>& layout_group) const;
-  void fill_layout_field_details(const Glib::ustring& parent_table_name, type_mapLayoutGroupSequence& sequence) const;
+  void fill_layout_field_details(const Glib::ustring& parent_table_name, type_list_layout_groups& groups) const;
 
 
 
@@ -265,6 +296,14 @@ public:
   sharedptr<Report> get_report(const Glib::ustring& table_name, const Glib::ustring& report_name) const;
   void remove_report(const Glib::ustring& table_name, const Glib::ustring& report_name);
 
+  //Print Layouts are precisely positioned layouts for printing to a printer:
+  typedef std::list<Glib::ustring> type_listPrintLayouts;
+  type_listPrintLayouts get_print_layout_names(const Glib::ustring& table_name) const;
+  void remove_all_print_layouts(const Glib::ustring& table_name);
+  void set_print_layout(const Glib::ustring& table_name, const sharedptr<PrintLayout>& print_layout);
+  sharedptr<PrintLayout> get_print_layout(const Glib::ustring& table_name, const Glib::ustring& print_layout_name) const;
+  void remove_print_layout(const Glib::ustring& table_name, const Glib::ustring& print_layout_name);
+
   void set_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name, const Gnome::Gda::Value& primary_key_value);
   void forget_layout_record_viewed(const Glib::ustring& table_name);
   Gnome::Gda::Value get_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name) const;
@@ -274,30 +313,29 @@ public:
 
   // Used by Relationship Overview dialog to preserve table locations accross instantiations:
     
-    /**
-     * Retrieve the x and y coordinates for the given table position.
-     * 
-     * @param table_name The name of the table to query.
-     * @param x The x coordinate of the table position.
-     * @param y The y coordinate of the table position.
-     * @return false if the table does not have any 
-     */
-    bool get_table_overview_position ( const Glib::ustring &table_name, float &x, float &y ) const;
+  /** Retrieve the x and y coordinates for the given table position.
+   * 
+   * @param table_name The name of the table to query.
+   * @param x The x coordinate of the table position.
+   * @param y The y coordinate of the table position.
+   * @return false if the table does not have any 
+   */
+  bool get_table_overview_position ( const Glib::ustring &table_name, float &x, float &y ) const;
     
-    /**
-     * Set the position of a table in the relationship overview dialog.
-     * 
-     * @param table_name The name of the table to modify.
-     * @param x The x coordinate of the table position.
-     * @param y The y coordinate of the table position.
-     */
-    void set_table_overview_position ( const Glib::ustring &table_name, float x, float y );
+  /** Set the position of a table in the relationship overview dialog.
+   * 
+   * @param table_name The name of the table to modify.
+   * @param x The x coordinate of the table position.
+   * @param y The y coordinate of the table position.
+   */
+  void set_table_overview_position ( const Glib::ustring &table_name, float x, float y );
     
   enum userLevelReason
   {
     USER_LEVEL_REASON_UNKNOWN,
     USER_LEVEL_REASON_FILE_READ_ONLY,
-    USER_LEVEL_REASON_DATABASE_ACCESS_LEVEL
+    USER_LEVEL_REASON_DATABASE_ACCESS_LEVEL,
+    USER_LEVEL_REASON_OPENED_FROM_BROWSE
   };
 
   /**
@@ -306,7 +344,7 @@ public:
    */
   AppState::userlevels get_userlevel(userLevelReason& reason) const;
 
-   AppState::userlevels get_userlevel() const;
+  AppState::userlevels get_userlevel() const;
 
   /** This is transitory information, not saved to disk.
    */
@@ -319,71 +357,64 @@ public:
 
   void set_parent_window(Gtk::Window* window);
 
+  Glib::ustring build_and_get_contents() const;
+
   static sharedptr<TableInfo> create_table_system_preferences();
   static sharedptr<TableInfo> create_table_system_preferences(type_vecFields& fields);
   static sharedptr<Relationship>  create_relationship_system_preferences(const Glib::ustring& table_name);
   static bool get_relationship_is_system_properties(const sharedptr<const Relationship>& relationship);
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, int value);
+  ///If value is equal to the default then no attribute will be set, to save text space in the XML file.
+  static void set_node_attribute_value_as_decimal(xmlpp::Element* node, const Glib::ustring& strAttributeName, guint value, guint value_default = 0);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   // This is required by the report builder, so it cannot be disabled
   // in client only mode
   static void set_node_attribute_value_as_decimal_double(xmlpp::Element* node, const Glib::ustring& strAttributeName, double value);
 
-
-
 protected:
 
   //Overrides:
-  virtual bool load_after();
+
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual bool save_before();
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
-  void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring& table_name, const sharedptr<LayoutGroup>& group);
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  void save_before_layout_group(xmlpp::Element* node, const sharedptr<const LayoutGroup>& group);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
-  void load_after_sort_by(const xmlpp::Element* node, const Glib::ustring& table_name, LayoutItem_GroupBy::type_list_sort_fields& list_fields);
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
+  void save_before_layout_group(xmlpp::Element* node, const sharedptr<const LayoutGroup>& group, bool with_print_layout_positions = false);
   void save_before_sort_by(xmlpp::Element* node, const LayoutItem_GroupBy::type_list_sort_fields& list_fields);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
-
-  void load_after_layout_item_usesrelationship(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<UsesRelationship>& item);
-  void load_after_layout_item_field(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<LayoutItem_Field>& item);
-  void load_after_layout_item_field_formatting(const xmlpp::Element* element, FieldFormatting& format, Field::glom_field_type field_type, const Glib::ustring& table_name, const Glib::ustring& field_name);
-
-  void load_after_translations(const xmlpp::Element* element, TranslatableItem& item);
-
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   void save_before_layout_item_usesrelationship(xmlpp::Element* nodeItem, const sharedptr<const UsesRelationship>& item);
   void save_before_layout_item_field(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem_Field>& item);
-  void save_before_layout_item_field_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, Field::glom_field_type field_type);
+  void save_before_layout_item_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID);
 
   void save_before_translations(xmlpp::Element* nodeItem, const TranslatableItem& item);
+  void save_before_print_layout_position(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem>& item);
 
   void save_changes();
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
+  virtual bool load_after();
+  void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring& table_name, const sharedptr<LayoutGroup>& group, bool with_print_layout_positions = false);
+  void load_after_sort_by(const xmlpp::Element* node, const Glib::ustring& table_name, LayoutItem_GroupBy::type_list_sort_fields& list_fields);
+  void load_after_layout_item_usesrelationship(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<UsesRelationship>& item);
+  void load_after_layout_item_field(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<LayoutItem_Field>& item);
+  void load_after_layout_item_formatting(const xmlpp::Element* element, FieldFormatting& format, Field::glom_field_type field_type = Field::TYPE_INVALID, const Glib::ustring& table_name = Glib::ustring(), const Glib::ustring& field_name = Glib::ustring());
+
+  void load_after_translations(const xmlpp::Element* element, TranslatableItem& item);
+  void load_after_print_layout_position(const xmlpp::Element* nodeItem, const sharedptr<LayoutItem>& item);
+
   void on_app_state_userlevel_changed(AppState::userlevels userlevel);
 
   void fill_translatable_layout_items(const sharedptr<LayoutGroup>& group, type_list_translatables& the_list);
 
-  static bool get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true);
-#endif // !GLOM_ENABLE_CLIENT_ONLY
 
-  static guint get_node_attribute_value_as_decimal(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
+  /// If the attribute is not there, then the default will be returned.
+  static bool get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value_default = false);
+  static guint get_node_attribute_value_as_decimal(const xmlpp::Element* node, const Glib::ustring& strAttributeName, guint value_default = 0);
   static double get_node_attribute_value_as_decimal_double(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
-
   static float get_node_attribute_value_as_float(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
+  static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true, bool value_default = false);
   static void set_node_attribute_value_as_float( xmlpp::Element* node, const Glib::ustring& strAttributeName, float value );
   static void set_node_attribute_value_as_value(xmlpp::Element* node, const Glib::ustring& strAttributeName, const Gnome::Gda::Value& value, Field::glom_field_type field_type);
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -392,8 +423,6 @@ protected:
   Glib::ustring get_child_text_node(const xmlpp::Element* node, const Glib::ustring& child_node_name) const;
   void set_child_text_node(xmlpp::Element* node, const Glib::ustring& child_node_name, const Glib::ustring& text);
 
-  void append_newline(xmlpp::Element* parent_node);
-
   AppState m_app_state;
   type_signal_userlevel_changed m_signal_userlevel_changed;
 
@@ -402,6 +431,8 @@ protected:
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
   Glib::ustring m_connection_server, m_connection_user, m_connection_database;
+  int m_connection_port; //0 means any port. Ignored when self-hosting (which may use a different port each time).
+  bool m_connection_try_other_ports; //Set to false for self-hosted or browsed-from-network documents.
 
   class LayoutInfo
   {
@@ -409,7 +440,7 @@ protected:
     Glib::ustring m_layout_name;
     Glib::ustring m_parent_table;
 
-    type_mapLayoutGroupSequence m_layout_groups;
+    type_list_layout_groups m_layout_groups;
   };
 
   class DocumentTableInfo
@@ -433,6 +464,9 @@ protected:
     typedef std::map< Glib::ustring, sharedptr<Report> > type_reports; //map of report names to reports
     type_reports m_reports;
 
+    typedef std::map< Glib::ustring, sharedptr<PrintLayout> > type_print_layouts; //map of print layout names to print layouts
+    type_print_layouts m_print_layouts;
+
     Glib::ustring m_example_rows;
 
     //Per-session, not saved in document:
@@ -468,7 +502,9 @@ protected:
   bool m_is_example;
   guint m_document_format_version;
 
-  Gtk::Window* m_parent_window; //Needed by BusyCursor.(gtype_0)
+  bool m_opened_from_browse;
+
+  Gtk::Window* m_parent_window; //Needed by BusyCursor.
 };
 
 //The base View for this document;
