@@ -80,8 +80,10 @@ Glib::ustring locale_language_id(const Glib::ustring& locale_id);
 
 Glib::ustring create_local_image_uri(const Gnome::Gda::Value& value);
 
+/** Get a decimal text representation of the number,
+ * in the current locale.
+ */
 Glib::ustring string_from_decimal(guint decimal);
-guint decimal_from_string(const Glib::ustring& str);
 
 /** Create an appropriate title for an ID string.
  * For instance, date_of_birth would become Date Of Birth.
@@ -97,10 +99,16 @@ Glib::ustring string_remove_suffix(const Glib::ustring& str, const Glib::ustring
 
 int dialog_run_with_help(Gtk::Dialog* dialog, const Glib::ustring& id = Glib::ustring());
 
+/** This is a replacement for gnome_help_display(), 
+ * to avoid the libgnome dependency.
+ * TODO: GTK+ should have a function for this soon.
+ */
 void show_help(const Glib::ustring& id = Glib::ustring());
 
 void show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent, Gtk::MessageType message_type);
 
+void show_window_until_hide(Gtk::Window* window);
+
 } //namespace Utils
 
 } //namespace Glom
