@@ -46,8 +46,10 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Button_Nav_Next(Gtk::Stock::GO_FORWARD),
   m_Button_Nav_Last(Gtk::Stock::GOTO_LAST),
   m_bDoNotRefreshRelated(false),
-  m_ignore_signals(true),
-  m_design_mode(false)
+  m_ignore_signals(true)
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  ,m_design_mode(false)
+#endif
 {
   m_layout_name = "details";
 
@@ -76,9 +78,10 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_NONE); //SHADOW_IN is Recommended by the GNOME HIG, but looks odd.
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_HBox_Sidebar.pack_start (m_Dragbar, Gtk::PACK_SHRINK);
+  m_HBox_Sidebar.pack_start(m_Dragbar, Gtk::PACK_SHRINK);
   m_Dragbar.hide();
 #endif
+
   m_HBox_Sidebar.pack_start(m_ScrolledWindow);
   m_ScrolledWindow.add(m_FlowTable);
   // The FlowTable does not support native scrolling, so gtkmm adds it to a
@@ -149,7 +152,7 @@ void Box_Data_Details::set_primary_key_value(const Gtk::TreeModel::iterator& /*
 
 void Box_Data_Details::set_found_set_from_primary_key_value()
 {
-  if (!m_primary_key_value.is_null())
+  if(!m_primary_key_value.is_null())
   {
     m_found_set.m_where_clause = "\"" + m_table_name + "\".\"" + m_field_primary_key->get_name() + 
       "\" = " + m_field_primary_key->sql(m_primary_key_value);
@@ -207,16 +210,9 @@ void Box_Data_Details::create_layout()
       m_FlowTable.add_layout_group(*iter);
     }
   }
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   m_FlowTable.set_design_mode(m_design_mode);
-  if (m_design_mode)
-  {
-    m_Dragbar.show();
-  }
-  else
-  {
-    m_Dragbar.hide();
-  }
 #endif
 }
 
@@ -996,6 +992,15 @@ void Box_Data_Details::prepare_layout_dialog(Dialog_Layout* dialog)
 {
   dialog->set_document(m_layout_name, get_document(), m_table_name, m_FieldsShown); //TODO: Use m_TableFields?
 }
+
+void Box_Data_Details::show_layout_toolbar (bool show)
+{
+  if(show)
+    m_Dragbar.show();
+  else
+    m_Dragbar.hide();
+}
+
 #endif // !GLOM_ENABLE_CLIENT_ONLY
 
 } //namespace Glom
